/***************************************************************************
 *            DBInterface.h
 *
 *  Sat May  3 01:39:15 2003
 *  Copyright  2003  Ricardo Markiewicz
 *  gazer@daemonbsd.com.ar
 ****************************************************************************/
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 

#if !defined(DBINTERFACE_H)
#define DBINTERFACE_H

#include <fox/fx.h>
#include <mysql/mysql.h>
#include <string.h>

/* I think that this sucks :-), but works for now! */
#define LAST 999999

class DBFieldData {
public:
	DBFieldData() {}
	~DBFieldData() {}

	FXString name;
	unsigned int type;
	unsigned int length;
	unsigned int flags;
	FXString def_val;
	
};

/** This is a MySQL FOX Widget.
  *
  * Basically is a FOX Widget that manage queries in an MySQL table. At this moment can handle
  * SELECTs, UPDATEs and DELETEs queries in a single table, and can have a relation with another
  * DBInterface with automatic update!.
  * In the other hand, you have some DBWidgets, that can be connected with the DBInterface to
  * show a field value of one record, all with one line of code :-)
  */
class DBInterface:public FXVerticalFrame {
	FXDECLARE(DBInterface)
public:
	DBInterface(FXComposite *p,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_SPACING,FXint pr=DEFAULT_SPACING,FXint pt=DEFAULT_SPACING,FXint pb=DEFAULT_SPACING,FXint hs=DEFAULT_SPACING,FXint vs=DEFAULT_SPACING);
	virtual ~DBInterface();
	virtual void create();

	enum {
		ID_BAR=FXVerticalFrame::ID_LAST,
		ID_LAST
	};

	/** Connect with a MySQL Server
     *
     *  \param host IP or DNS Name of the server
     *  \param user Username of MySQL server
     *  \param password Password for username
     *  \param dbname Database name
     */
	void connect(char *host, char *user, char *password, char *dbname);
	/** Set the name of the table to handle
     *
     *  This function read all data of the fields and records in the table, then, the magic
     *  is done!
     */
	void setTable(FXString);
	/** Get the table name */
	FXString getTableName() { return table_name; }

	/** Get the current value of a field in the corrent record.
	 *
	 *  This is deprecated!! ... now i use the DataTarget to update the records!
	 */
	FXString getField(FXString);
	
	/* TO BE REMOVED! */
	int getNumTables() { return num_tables; }
	FXString getTable(int i);

	/** Return the DataTarget of a field
	 *
	 *  When a DBWidget has a field assigned, it request the datatarget of that field
	 *  for automatic update when the record change in the DBInterface
	 */
	FXDataTarget *getFieldTarget(const FXString &s);
	/** Return the current ID DataTarget
	 *
	 *  The ID DataTarget is need when 2 DBInterfaces has a relation. So, when the MASTER
	 *  change the current record, the SLAVE change at the same time!
	 *
	 *  \see relateWith
	 */
	FXDataTarget *getIDTarget() { return &id_target; }

	/** Made a relation with another DBInterface
	 *
	 *  This functios tell the current DB Interface (SLAVE) that the query depend of the state of
	 *  other DBInterface (MASTER), and must be updated when the MASTER update. 
	 *  I try to make this generic, so, you can relate what you want.
	 *
	 *  When the query is related, the MySQL stament is something like :
	 *    SELECT t1.* FROM this_table AS t1, related_table AS t2 WHERE t1.field_slave=t2.field_master AND t2.field_master=current_relation_id
	 *
	 *  \param db The MASTER DBInterface to follow
	 *  \param slave_field Field of the SLAVE DB
	 *  \param master_field Field of the MASTER DB
	 */
	void relateWith(DBInterface *db, const FXString &slave_field, const FXString &master_field);

	/* Database operators */
	/** Init a INSERT query
	 *
	 *  This function blank all fields_data and let the user fill the data of a new record. When the user
	 *  finish, saveEntry must be save to execute a MySQL query for save the data in the DB
	 *  \see saveEntry
	 */
	void newEntry();
	/** Complete a save or update query
	 *
	 *  Depending of the actual state (SAVE or UPDATE) create an execute a MySQL query for sabe the actual data
	 */
	void saveEntry();
	/** Same as newEntry but for UPDATE query */
	void updateEntry();
	/** Cancel the current update or save action */
	void cancelEntry();
	void deleteFirstEntry();
	/** Delete the current field from the MySQL DB */
	void deleteCurrentEntry();
	void deleteLastEntry();

	/* Events */
	long onValueChange(FXObject*,FXSelector,void*);
	long onRelationChange(FXObject *obj, FXSelector sel, void *ptr);
protected:
	void moveTo(unsigned int i);

	/* ******************* */
	enum {NEW, UPDATE, NONE};
	int state; // state of the DB (see emun above)
	DBInterface() {}
	FXLabel *info; // for verbose msg to the user
	FXScrollBar *bar; // This bar is to navigate the DB :-), do magic ...
	MYSQL mysql;
	FXString table_name;
	int current_id; // This has the same value of bar->getPosition, but is needed for the FXDataTarget in a relation

	/* ID vectors : Save the ID field of each record */
	int identifiers[100]; // si lo hao dinamico se palma :-(
	int num_id;

	/* Realtion info, if any */
	FXDataTarget id_target; // DataTarget for automatic update
	bool is_related; // flag
	int current_relation_id; // if this table is related, this save the actual id of the related field
	FXString fields_related[2]; // 0==this db field, 1==related db field
	DBInterface *db_related; // Pointer to the DB relation
	
	/* Field info & data */
	int fields_count;
	DBFieldData fields_info[100]; // si lo hago dinamico se palma :-(
	FXString fields_data[100]; // When the user change the current_id, this vector sabe the SELECT result
	FXString fields_update[100]; // After update, this vector saves the actual state of the fields
	FXDataTarget target_data[100]; // DataTarget for every field for automatic visual update
	
	/*Not done or to be removed */
	FXString tables[100];
	int num_tables;
};

#endif // !defined(DBINTERFACE_H)
