/***************************************************************************
 *            DBInterface.cpp
 *
 *  Sat May  3 01:38:30 2003
 *  Copyright  2003  Ricardo Markiewicz
 *  gazer@daemonbsd.com.ar
 ****************************************************************************/
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 

#include "DBInterface.h"


FXDEFMAP(DBInterface) DBInterfaceMap[]={
  FXMAPFUNC(SEL_COMMAND,DBInterface::ID_BAR,DBInterface::onValueChange),
  FXMAPFUNC(SEL_COMMAND,DBInterface::ID_SETINTVALUE,DBInterface::onRelationChange)
};

FXIMPLEMENT(DBInterface, FXVerticalFrame, DBInterfaceMap, ARRAYNUMBER(DBInterfaceMap));

DBInterface::DBInterface(FXComposite *p,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb,FXint hs,FXint vs):FXVerticalFrame(p, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
{
	FXHorizontalFrame *hf;

	hf = new FXHorizontalFrame(this, LAYOUT_FILL_X);

	//new FXButton(hf, "<<");
	bar = new FXScrollBar(hf, this, ID_BAR, LAYOUT_FILL_X|LAYOUT_FILL_Y|SCROLLBAR_HORIZONTAL);
	bar->disable();
	//new FXButton(hf, ">>");
	info = new FXLabel(this, "Info", NULL, LAYOUT_FILL_X|LAYOUT_FILL_Y);

	for(int i=0; i<100; i++)
		target_data[i].connect(fields_data[i]);

	current_id = 0;
	is_related = false;
	db_related = NULL;
	id_target.connect(current_id);
	state = NONE;
}

DBInterface::~DBInterface()
{
}

void DBInterface::create()
{
	FXVerticalFrame::create();

	show();
}

void DBInterface::connect(char *host, char *user, char *password, char *dbname)
{
	if (mysql_connect(&mysql, host, user, password) == NULL) {
		info->setText(mysql_error(&mysql));
	} else {
		mysql_select_db(&mysql, dbname);
		info->setText("Conected to MySQL server ....");

		// Load the tables record
		MYSQL_RES *result;
		MYSQL_ROW row;
			
		result = mysql_list_tables(&mysql, NULL);
		int i=0;
		while ((row = mysql_fetch_row(result))) {
			tables[i++] = row[0];
		}
		mysql_free_result(result);
		num_tables = i;
	}
}

void DBInterface::setTable(FXString t)
{
	FXString query;
	table_name = t;

	MYSQL_RES *result;
	MYSQL_ROW row;

	// Query for the items
	if (is_related) {
		query = FXString("SELECT count(t1.id) FROM ")+table_name+" AS t1,";
		query += db_related->getTableName()+" AS t2 WHERE t1.";
		query += fields_related[0];
		query += "=t2.";
		query += fields_related[1];
		query += " AND t2.";
		query += fields_related[1];
		query += "=";
		query += FXStringVal(current_relation_id);
	} else {
		query = FXString("SELECT count(id) FROM ")+table_name;
	}
	if (!mysql_query(&mysql, query.text())) {
		// Get the result
		result = mysql_use_result(&mysql);
		if (result) {
			// Read the record-count from the DB
			row=mysql_fetch_row(result);

			// Create the data arrays
			num_id = atoi(row[0]);

			// Set the range for the bar
			bar->setRange(num_id);
			// Free the result
			mysql_free_result(result);
							
			// Now, read all the IDs from the DB
			unsigned int i=0;
			if (is_related) {
				query = FXString("SELECT t1.* FROM ")+table_name+" AS t1,";
				query += db_related->getTableName()+" AS t2 WHERE t1.";
				query += fields_related[0];
				query += "=t2.";
				query += fields_related[1];
				query += " AND t2.";
				query += fields_related[1];
				query += "=";
				query += FXStringVal(current_relation_id);
			} else {
				query = FXString("SELECT * FROM ")+table_name;
			}
			if (!mysql_query(&mysql, query.text())) {
				result = mysql_use_result(&mysql);
				// Save fields info ...
				unsigned int num_fields, id_column;
				MYSQL_FIELD *fields;
				num_fields = mysql_num_fields(result);
				fields = mysql_fetch_fields(result);
				//fields_info = new DBFieldData[num_fields];
				for(i=0; i < num_fields; i++) {
					fields_info[i].name = fields[i].name;
					fields_info[i].type = fields[i].type;
					fields_info[i].length = fields[i].length;
					fields_info[i].def_val = fields[i].def;
					fields_info[i].flags = fields[i].flags;
					fields_data[i] = "";
					if (fields_info[i].name == "id") id_column = i;
				}
				fields_count = num_fields;

				// Save the ID of each record in the database
				i=0;
				while ((row=mysql_fetch_row(result))) {
					identifiers[i++] = atoi(row[id_column]);
				}
				mysql_free_result(result);
			}

			bar->setPosition(0);
			bar->enable();
		} else {
			info->setText(mysql_error(&mysql));
		}
	} else {
		info->setText(mysql_error(&mysql));
	}

	moveTo(0);
}

FXString DBInterface::getField(FXString s)
{
	for(int i=0; i<fields_count; i++) {
		if (fields_info[i].name == s) {
			return fields_data[i];
		}
	}
	return "";
}


long DBInterface::onValueChange(FXObject *o,FXSelector sel,void *ptr)
{
	FXString query;
	MYSQL_RES *result;
	MYSQL_ROW row;

	info->setText(table_name+FXString(" : ")+FXStringVal(bar->getPosition()+1)+FXString("/")+FXStringVal(bar->getRange()));
	query = FXString("SELECT * FROM ")+table_name+FXString(" WHERE id=")+FXStringVal(identifiers[bar->getPosition()]);
	if (!mysql_query(&mysql, query.text())) {
		result = mysql_use_result(&mysql);
		row = mysql_fetch_row(result);
		for(int i=0; i<fields_count; i++) {
			fields_data[i]=row[i];
		}
		mysql_free_result(result);
	} else {
		info->setText( mysql_error(&mysql) );
	}

	if (getTarget()) getTarget()->handle(this, MKUINT(getSelector(),SEL_COMMAND), ptr);
	current_id = identifiers[bar->getPosition()];
	return 0;
}

FXString DBInterface::getTable(int i)
{
	if ((i>=0) && (i<num_tables))
		return tables[i];

	return "";
}

FXDataTarget *DBInterface::getFieldTarget(const FXString &s)
{
	for(int i=0; i<fields_count; i++) {
		if (fields_info[i].name == s) {
			return &target_data[i];
		}
	}
	return NULL;
}

void DBInterface::newEntry()
{
	state = NEW;
	bar->disable();
	for(int i=0; i<fields_count; i++)
		fields_data[i] = "";
}

void DBInterface::saveEntry()
{
	FXString query;
	int i;
	switch (state) {
		case NEW:
			query = "INSERT INTO "+table_name+" (";
			for(i=0; i<fields_count-1; i++) {
				query += fields_info[i].name+",";
			}
			query += fields_info[i].name+") VALUES (";
			for(i=0; i<fields_count; i++) {
				// Autoincrements fields are inserter as NULL!
				if (fields_info[i].flags & AUTO_INCREMENT_FLAG) {
					query += "NULL";
				} else {
					// Strings types need quotes
					if (IS_NUM(fields_info[i].type)) {
						if (fields_data[i] == "") {
							query += FXStringVal(0);
						} else {
							query += fields_data[i];
						}
					} else {
						query += FXString("\"")+fields_data[i]+"\"";
					}
				}
				if (i == fields_count-1)
					query += ")";
				else
					query += ",";
			}
			// Execure the query!
			if (mysql_query(&mysql, query.text())) {
				info->setText( mysql_error(&mysql) );
			} else {
				info->setText( "New Record added !" );
			}
			setTable(table_name);
			bar->enable();
			moveTo(LAST);
		break;
		case UPDATE:
			query = "UPDATE "+table_name+" SET ";
			int update_count=0;
			int put_comma=0;
			for(i=0; i<fields_count; i++) {
				// Check if the field has changed
				if (fields_update[i] != fields_data[i]) {
					if (put_comma) {
						query += ", ";
						put_comma=0;
					}
					query += fields_info[i].name+"=";
					if (IS_NUM(fields_info[i].type)) {
						query += fields_data[i];
					} else {
						query += FXString("\"")+fields_data[i]+"\"";
					}
					if (i != fields_count-1)
						put_comma=1;
					update_count++;
				}
			}
			query += " WHERE id="+FXStringVal(identifiers[bar->getPosition()]);
			if (update_count == 0)
				info->setText("Nothing Change!");
			else {
				if (mysql_query(&mysql, query.text())) {
					info->setText( mysql_error(&mysql) );
				} else {
					info->setText( "Record Updated !" );
				}
			}
			state = NONE;
			bar->enable();
			moveTo(bar->getPosition());
	}
	// Trick for reloading the MySQL table info!
	state = NONE;
}

void DBInterface::updateEntry()
{
	if (num_id == 0) return;
	state = UPDATE;
	bar->disable();
	// Save the actual values of the register
	for(int i=0; i<fields_count; i++)
		fields_update[i] = fields_data[i];
}

void DBInterface::deleteFirstEntry()
{
}

void DBInterface::deleteCurrentEntry()
{
	if (num_id == 0) return;
	FXString query = "DELETE FROM "+table_name+FXString(" WHERE id=")+FXStringVal(identifiers[bar->getPosition()]);
	if (mysql_query(&mysql, query.text())) {
		info->setText( mysql_error(&mysql) );
	} else {
		info->setText( "Record Updated !" );
		setTable(table_name);
		moveTo(0);
	}
}

void DBInterface::deleteLastEntry()
{
}

void DBInterface::cancelEntry()
{
	state = NONE;
	bar->enable();
	moveTo(0);
}

void DBInterface::moveTo(unsigned int i)
{
	if (num_id == 0) {
		info->setText(table_name+" : Table empty");
		return;
	}
	if (i > (unsigned int)bar->getRange()) i = bar->getRange();

	bar->setPosition(i);
	handle(this, MKUINT(bar->getSelector(), SEL_COMMAND), NULL);
}

void DBInterface::relateWith(DBInterface *db,  const FXString &field1, const FXString &field2)
{
	is_related = !(db == NULL);
	db_related = db;

	if (is_related) {
		fields_related[0] = field1;
		fields_related[1] = field2;
		setTarget( db->getIDTarget() );
		setSelector(FXDataTarget::ID_VALUE);
	}
}

long DBInterface::onRelationChange(FXObject *obj, FXSelector sel, void *ptr)
{
	int i = *((int *)ptr);
	if (current_relation_id != i) {
		current_relation_id = i;
		setTable(table_name);
	}
	return 0;
}
